*** THE GLOBAL DOLLAR CYCLE ***

* PANEL LOCAL PROJECTIONS: Excess bond premium (EBP) shocks *

clear all
set more off
cd "$masterpath"
set scheme s1color
graph drop _all

** read data **
use "data/lp/lp_data.dta", clear
xtset cid yq

/////////////////////////////////////////////////////////////////////
///////////////////// LOCAL PROJECTION: PREP ////////////////////////
/////////////////////////////////////////////////////////////////////

* generate change in the price level
cap drop pp
cap drop pi
gen pp = 100 * (p - l4.p) / l4.p		// % yoy change
gen pi = 100 * (cpi - l.cpi) / l.cpi	// % yoy change
replace pp = . if pp > 50 & pp < .
replace pi = . if pi > 50 & pi < .

* data in levels: log difference (quarterly)
local data_l rgdp rcons rgc rinv p cpi xr neer tot aeusd_g aeusd_avg broadusd_g ncredit_lc rexp rimp pexp pimp equity
foreach var in `data_l' {
    gen d`var' = 100 * log(`var' / l.`var')
}

* data in percentages: first difference in levels
local data_p fci rpol ffr ltrates kgdp shadow embi ebp pp pi
foreach var in `data_p' {
    gen d`var' = d.`var'
}

* US monetary policy controls
* replace shadow rate with FFR before 2009
gen dmon = dshadow
replace dmon = dffr if year(dofq(yq)) <= 2008

* divide response variables into three buckets:
* national accounts: read gdp, private and government consumption, investment, 
* exports and imports
local res_na rgdp rcons rgc rinv rexp rimp

* price: gdp deflator, CPI, exchange rate, export/import prices, tot
local res_price pp pi xr pexp pimp tot

* interest rate, nominal credit
local res_int ncredit_lc equity rpol embi

local res_all `res_na' `res_price' `res_int'

** define variables included in the regressions **

* global factors
local r factor_em dmon dfci

* local controls + ae dollar index
local q drgdp dxr drpol daeusd_g

* shock: dollar index against advanced economies (EOQ)
local x debp

* rescale shock size to 2.5%
replace debp = debp / 2.5

* combine them
local w `r' `x' `q'

** Generate forward value ***
qui forval h=0/12 {
	foreach y of local res_all {
		cap drop `y'`h'
        if !inlist("`y'", "kgdp", "rpol", "ltrates", "embi", "pp", "pi") {
            gen `y'`h' = 100 * log(f`h'.`y' / l.`y')        // cumulative IRF
        }
        if inlist("`y'", "kgdp", "rpol", "ltrates", "embi", "pp", "pi") {
            gen `y'`h' = f`h'.`y' - l.`y'
        }	
	}
	local y aeusd_g
	gen `y'`h' = 100 * log(f`h'.`y' / l.`y')
}

xtset cid yq
keep if year(dofq(yq)) <= 2019


/////////////////////////////////////////////////////////////////////
///////////////////// LOCAL PROJECTION: LOOP ////////////////////////
/////////////////////////////////////////////////////////////////////

******* NATIONAL ACCOUNTS *******
foreach y of local res_na {

	cap drop b_* se_* Horizon
	cap drop h0_* l0_* h1_* l1_*
	gen b_`y'=.
	gen h0_`y'=.
	gen l0_`y'=.
	gen h1_`y'=.
	gen l1_`y'=.
	gen se_`y'=.

	gen Horizon = _n-1 if _n<=12+1

	eststo clear
	
	local max 12
	
	// Regression Loop
	forvalues h=0/`max' {	

		reghdfe `y'`h' `r' `x' l(1/4).(`w') l(1/4).d`y', vce(robust) a(cid)
        eststo h`h'
			
		qui replace  b_`y'    =  _b[`x']                 if _n == `h'+1
		qui replace se_`y'    = _se[`x']                 if _n == `h'+1	
		qui replace  l0_`y'    = _b[`x']- _se[`x']  if _n == `h'+1 	
		qui replace  h0_`y'    = _b[`x']+ _se[`x']  if _n == `h'+1
		qui replace  l1_`y'    = _b[`x']- _se[`x']*1.645  if _n == `h'+1 	
		qui replace  h1_`y'    = _b[`x']+ _se[`x']*1.645  if _n == `h'+1
		
	}
		
	****** Output tables ******

    ** dollar **
    noisily esttab ///
		h0 h3 h6 h9 h12  ///
		, compress keep(`x') b(%8.2f) se(%8.2f) title("`y'") mtitles nonum label /// 
		star(* 0.10 ** 0.05 *** 0.01)

	****** Export IRFs ******
	preserve
	keep if Horizon <= 12

    if "`y'" == "rgdp" local tname "real GDP"
    if "`y'" == "rcons" local tname "private consumption"
    if "`y'" == "rgc" local tname "government consumption"
    if "`y'" == "rinv" local tname "gross capital formation"
    if "`y'" == "rexp" local tname "real export"
    if "`y'" == "rimp" local tname "real import"

    local ytitle "100 x log"

	cap drop Zero
	gen Zero=0
	qui twoway ///
		(  rarea h1_ l1_ Horizon if Horizon~=. , fc(navy) fi(10) lcolor(white) lpattern(solid)  )  ///
		(  rarea h0_ l0_ Horizon if Horizon~=. , fc(ltblue) fi(100) lcolor(ltblue) lpattern(solid)  )  ///
		(  line  Zero    Horizon if Horizon~=. , lc(gray)  lw(medthin) lp(dash)                     )  ///		
		(  line  b_     Horizon if Horizon~=. , lc(ebblue)   lw(thick) lp(solid)                     ///
				xtitle("Quarter") xlab(0(4)12) ///
				ytitle("`ytitle'") ///
				legend(off)  /// 
																				          )  ///
		, title("`tname'") name(`y')
	*
	restore
	**

}

gr combine `res_na', row(2)
gr export  "$outputpath/ebp_na.pdf", replace


******* PRICES *******
foreach y of local res_price {

	cap drop b_* se_* Horizon
	cap drop h0_* l0_* h1_* l1_*
	gen b_`y'=.
	gen h0_`y'=.
	gen l0_`y'=.
	gen h1_`y'=.
	gen l1_`y'=.
	gen se_`y'=.

	gen Horizon = _n-1 if _n<=12+1

	eststo clear
	
	local max 12
	
	// Regression Loop
	forvalues h=0/`max' {	

		reghdfe `y'`h' `r' `x' l(1/4).(`w') l(1/4).d`y', vce(robust) a(cid)
        eststo h`h'
			
		qui replace  b_`y'    =  _b[`x']                 if _n == `h'+1
		qui replace se_`y'    = _se[`x']                 if _n == `h'+1	
		qui replace  l0_`y'    = _b[`x']- _se[`x']  if _n == `h'+1 	
		qui replace  h0_`y'    = _b[`x']+ _se[`x']  if _n == `h'+1
		qui replace  l1_`y'    = _b[`x']- _se[`x']*1.645  if _n == `h'+1 	
		qui replace  h1_`y'    = _b[`x']+ _se[`x']*1.645  if _n == `h'+1
		
	}
		
	****** Output tables ******

    ** dollar **
    noisily esttab ///
		h0 h3 h6 h9 h12  ///
		, compress keep(`x') b(%8.2f) se(%8.2f) title("`y'") mtitles nonum label /// 
		star(* 0.10 ** 0.05 *** 0.01)

	****** Export IRFs ******
	preserve
	keep if Horizon <= 12

    if "`y'" == "p" local tname "GDP deflator"
    if "`y'" == "cpi" local tname "consumer price"
	if "`y'" == "pp" local tname "GDP deflator change"
    if "`y'" == "pi" local tname "consumer price inflation"
    if "`y'" == "xr" local tname "exchange rate (LC/USD)"
    if "`y'" == "pexp" local tname "export price"
    if "`y'" == "pimp" local tname "import price"
    if "`y'" == "tot" local tname "terms of trade"

    local ytitle "100 x log"
	if inlist("`y'", "pp", "pi") local ytitle "pp yoy"

	cap drop Zero
	gen Zero=0
	qui twoway ///
		(  rarea h1_ l1_ Horizon if Horizon~=. , fc(navy) fi(10) lcolor(white) lpattern(solid)  )  ///
		(  rarea h0_ l0_ Horizon if Horizon~=. , fc(ltblue) fi(100) lcolor(ltblue) lpattern(solid)  )  ///
		(  line  Zero    Horizon if Horizon~=. , lc(gray)  lw(medthin) lp(dash)                     )  ///		
		(  line  b_     Horizon if Horizon~=. , lc(ebblue)   lw(thick) lp(solid)                     ///
				xtitle("Quarter") xlab(0(4)12) ///
				ytitle("`ytitle'") ///
				legend(off)  /// 
																				          )  ///
		, title("`tname'") name(`y')
	*
	restore
	**

}

gr combine `res_price', col(3)
gr export  "$outputpath/ebp_price.pdf", replace


******* CREDIT AND INTEREST RATES *******
foreach y of local res_int {

	cap drop b_* se_* Horizon
	cap drop h0_* l0_* h1_* l1_*
	gen b_`y'=.
	gen h0_`y'=.
	gen l0_`y'=.
	gen h1_`y'=.
	gen l1_`y'=.
	gen se_`y'=.

	gen Horizon = _n-1 if _n<=12+1

	eststo clear
	
	local max 12
	
	// Regression Loop
	forvalues h=0/`max' {	

		reghdfe `y'`h' `r' `x' l(1/4).(`w') l(1/4).d`y', vce(robust) a(cid)
        eststo h`h'
			
		qui replace  b_`y'    =  _b[`x']                 if _n == `h'+1
		qui replace se_`y'    = _se[`x']                 if _n == `h'+1	
		qui replace  l0_`y'    = _b[`x']- _se[`x']  if _n == `h'+1 	
		qui replace  h0_`y'    = _b[`x']+ _se[`x']  if _n == `h'+1
		qui replace  l1_`y'    = _b[`x']- _se[`x']*1.645  if _n == `h'+1 	
		qui replace  h1_`y'    = _b[`x']+ _se[`x']*1.645  if _n == `h'+1
		
	}
		
	****** Output tables ******

    ** dollar **
    noisily esttab ///
		h0 h3 h6 h9 h12  ///
		, compress keep(`x') b(%8.2f) se(%8.2f) title("`y'") mtitles nonum label /// 
		star(* 0.10 ** 0.05 *** 0.01)

	****** Export IRFs ******
	preserve
	keep if Horizon <= 12

    if "`y'" == "ncredit_lc" local tname "nominal credit (LC)"
    if "`y'" == "rpol" local tname "policy rate"
    if "`y'" == "ltrates" local tname "long-term rate (LC)"
    if "`y'" == "embi" local tname "EMBI spread"
	if "`y'" == "equity" local tname "equity price (LC)"
    
    local ytitle "pp"
    if "`y'" == "ncredit_lc" local ytitle "100 x log"
	if "`y'" == "equity" local ytitle "100 x log"

	cap drop Zero
	gen Zero=0
	qui twoway ///
		(  rarea h1_ l1_ Horizon if Horizon~=. , fc(navy) fi(10) lcolor(white) lpattern(solid)  )  ///
		(  rarea h0_ l0_ Horizon if Horizon~=. , fc(ltblue) fi(100) lcolor(ltblue) lpattern(solid)  )  ///
		(  line  Zero    Horizon if Horizon~=. , lc(gray)  lw(medthin) lp(dash)                     )  ///		
		(  line  b_     Horizon if Horizon~=. , lc(ebblue)   lw(thick) lp(solid)                     ///
				xtitle("Quarter") xlab(0(4)12) ///
				ytitle("`ytitle'") ylab(#3) ///
				legend(off)  /// 
																				          )  ///
		, title("`tname'") name(`y')
	*
	restore
	**
}

gr combine `res_int', col(2)
gr export  "$outputpath/ebp_int.pdf", replace


******* PUBLISHED VERSION: 6 selected variables *******
gr combine rgdp rinv pp xr equity rpol, col(3)
gr export  "$outputpath/ebp_pub.pdf", replace


graph drop _all